require_dependency "vulgata/application_controller"

module Vulgata
  class TeamController < ApplicationController
    before_action :authorize_admin, only: [:index, :update]
    before_action :authorize_viewer, only: [:show]

    def index
      @users = vulgata_users.page(params[:page]).per(20)

      @from_date = params["from_date"].present? ? params["from_date"].to_datetime.beginning_of_day : DateTime.current.beginning_of_month
      @to_date = params["to_date"].present? ? params["to_date"].to_datetime.end_of_day : DateTime.current.end_of_day

      ldvpu = Vulgata::TranslationStateVersion.where(status: Vulgata::TranslationState.statuses[:done])
      ldvpu = ldvpu.where("created_at >= ? AND created_at <= ?", @from_date, @to_date)
      ldvpu = ldvpu.group(:user_id, :vulgata_translation_state_id)
      ldvpu = ldvpu.select("MAX(id) AS id")
      @last_done_versions_per_user_and_t9n_state_sql = ldvpu.to_sql

      set_translated_items_per_user
      set_translated_words_per_user
    end

    def show
      @user = params[:user_type].constantize.find(params[:user_id])

      @total_translated = Vulgata::TranslationStateVersion.where(status: Vulgata::TranslationState.statuses[:done])
      @total_translated = @total_translated.where(user: @user)
      @total_translated = @total_translated.group(:vulgata_translation_state_id)
      @total_translated = @total_translated.select(:vulgata_translation_state_id)
      @total_translated = @total_translated.length || 0

      @translated_this_week = translated_in_time_period(DateTime.current.beginning_of_week)
      @translated_this_month = translated_in_time_period(DateTime.current.beginning_of_month)
      @translated_last_month = translated_in_time_period((DateTime.current - 1.month).beginning_of_month, DateTime.current.beginning_of_month)

      set_total_rejections @user
      set_overall_approval_rate

      versions_involving_user = Vulgata::TranslationStateVersion.where(user_id: params[:user_id])
      versions_involving_user = versions_involving_user.where.not(status: [Vulgata::TranslationState.statuses[:source], Vulgata::TranslationState.statuses[:pending], Vulgata::TranslationState.statuses[:in_progress]])
      versions_involving_user = versions_involving_user.select(:vulgata_translation_state_id)
      @translation_states = Vulgata::TranslationState.where(id: versions_involving_user).order(updated_at: :desc).page(params[:page]).per(10)

      locales = Vulgata::UserLocale.where(user: @user) || []
      @user_locales = locales.map(&:locale)
      @user_role = @user.vulgata_role
    end

    def update
      user = params[:user_type].constantize.find(params[:user_id])

      locales = params[:locales] || []
      transaction = ActiveRecord::Base.transaction do
        user.vulgata_locales = locales
        params[:role] == "none" ? user.vulgata_role = nil : user.vulgata_role = params[:role]
        user.save
      end

      respond_to do |format|
        if transaction
          format.html { redirect_to vulgata_user_path(user: user), notice: 'Successfully updated locales.' }
        else
          format.html { redirect_to vulgata_user_path(user: user), notice: 'Error updating locales' }
        end
      end
    end

    private

    def translated_in_time_period from, to = DateTime.now
      translated = Vulgata::TranslationStateVersion.where(status: Vulgata::TranslationState.statuses[:done])
      translated = translated.where(user: @user)
      translated = translated.where('created_at >= ? AND created_at < ?', from, to)
      translated = translated.group(:vulgata_translation_state_id)
      translated = translated.select(:vulgata_translation_state_id)
      translated = translated.length || 0
    end

    def set_total_rejections user
      rejected = Vulgata::TranslationState.statuses[:rejected]

      previously_rejected_translation_state_versions_sql = 
        Vulgata::TranslationStateVersion.where(status: rejected).to_sql
      
      prtswv = Vulgata::TranslationStateVersion.joins("INNER JOIN (#{previously_rejected_translation_state_versions_sql}) AS inn ON inn.vulgata_translation_state_id = vulgata_translation_state_versions.vulgata_translation_state_id")
      prtswv = prtswv.where("vulgata_translation_state_versions.id > inn.id")
      prtswv = prtswv.order("vulgata_translation_state_versions.id")
      prtswv = prtswv.select("vulgata_translation_state_versions.id, vulgata_translation_state_versions.vulgata_translation_state_id")
      previously_rejected_translation_states_with_versions = prtswv.to_sql
      
      after_rejection_versions = Vulgata::TranslationStateVersion.from("(#{previously_rejected_translation_states_with_versions}) as sub")
      after_rejection_versions = after_rejection_versions.group("sub.vulgata_translation_state_id")
      after_rejection_versions = after_rejection_versions.select("MIN(sub.id) AS pending_version_id_after_rejection")    
      
      rejected_translations_per_user = 
        Vulgata::TranslationStateVersion.where(id: after_rejection_versions).group(:user_type, :user_id).count

      @total_rejections = rejected_translations_per_user[[user.class.name, user.id]] || 0
    end

    def set_overall_approval_rate
      approval_rate = 0
      unless @total_translated == 0
        approval_rate = ((@total_translated - @total_rejections).to_f / @total_translated.to_f) * 100
      end

      @overall_approval_rate = approval_rate
    end

    def set_translated_items_per_user
      raw_translated_items_per_user = 
        Vulgata::TranslationStateVersion
          .select("user_id, user_type, count(*) AS items_translated")
          .where("id IN (#{@last_done_versions_per_user_and_t9n_state_sql})")
          .group(:user_id, :user_type)
      @translated_items_per_user = 
        raw_translated_items_per_user.each_with_object({}) { |r, h| h[r.user_type.to_s + r.user_id.to_s] = r.items_translated }
    end

    def set_translated_words_per_user
      raw_translated_words_per_user = 
        Vulgata::TranslationStateVersion
          .select("user_id, user_type, sum(word_count) AS words_translated")
          .where("id IN (#{@last_done_versions_per_user_and_t9n_state_sql})").group(:user_id, :user_type)
      @translated_words_per_user = 
        raw_translated_words_per_user.each_with_object({}) { |r, h| h[r.user_type.to_s + r.user_id.to_s] = r.words_translated }
    end

    def authorize_viewer
      redirect_to root_path unless params[:user_id].to_i == vulgata_current_user.id || vulgata_current_user.vulgata_admin?
    end
  end
end
