require_dependency "vulgata/application_controller"

module Vulgata
  class TranslationsController < ApplicationController
    before_action :authorize_all_role_holders, only: [:new, :create, :get_suggested_translation]
    before_action :authorize_admin_or_proofreader, only: [:show]
    before_action :authorize_admin, only: [:index]

    before_action :set_translation_state, only: [:create, :edit, :update]
    before_action :set_should_suggest_translation, only: [:new, :edit]

    # GET /translate
    def new
      set_translator_locales

      @translation_state = dequeue_translation_state_for_translating
      return if @translation_state.nil?
      @state_changed_to_pending = true

      set_translation_state_and_source_state
    end

    # GET /translation_states/1/translations/edit
    def edit
      authorize_admin_proofreader_or_assigned_translator @translation_state
      set_translation_state_and_source_state
    end

    # GET /admin/ModelName
    def index
      @resource = params[:resource]
      klass = @resource.constantize

      @items = klass.includes(:translation_states).where.not(vulgata_translation_states: { id: nil })
      @items = klass.vulgata_source_scope @items # query is scoped by the source item
      @items = klass.vulgata_where_like(params[:q], @items) if params[:q]
      @items_count = @items.references(:translation_states).group("vulgata_translation_states.status").count

      # filter by the selected status
      @items = @items.where(vulgata_translation_states: { status: Vulgata::TranslationState.statuses[params[:status]] }).references(:translation_states) if (params[:status].present? && params[:status] != "all")

      # sorting
      @items = case params[:sort]
         when 'updated_at_desc' then @items.order('vulgata_translation_states.updated_at DESC')
         when 'updated_at_asc' then @items.order('vulgata_translation_states.updated_at ASC')
         when 'priority_desc' then @items.order('vulgata_translation_states.priority DESC')
         when 'priority_asc' then @items.order('vulgata_translation_states.priority ASC')
         else @items.order('vulgata_translation_states.priority DESC, vulgata_translation_states.updated_at DESC')
      end

      # pagination
      @items = @items.page(params[:page]).per(10)
    end

    # GET /admin/ModelName/1
    def show
      @resource = params[:resource].constantize.find(params[:id])
    end

    # POST /translation_states/1/translations
    def create
      save_version_and_redirect_to new_translations_path
    end

    # PATCH /translation_states/1/translations
    def update
      authorize_admin_proofreader_or_assigned_translator @translation_state and return

      if params[:stop_translating].present?
        stop_translating
      else
        save_version_and_redirect_to translation_state_path(id: @translation_state)
      end
    end

    # GET /get_suggested_translation
    def get_suggested_translation
      suggest_translation params[:text], params[:from], params[:to]
    end

    private

    def stop_translating
      @translation_state.user = nil unless @translation_state.locked?
      @translation_state.status = Vulgata::TranslationState.statuses[:pending]

      respond_to do |format|
        if @translation_state.save
          format.html { redirect_to root_path, notice: 'Successfully stopped translating.' }
        else
          format.html { redirect_to translation_state_path(id: @translation_state), notice: 'Error stopping translating' }
        end
      end
    end

    def save_version_and_redirect_to redirect_path
      respond_to do |format|
        if @translation_state.save_translation_version(params[:resource], vulgata_current_user)
          after_translated_callback = @translation_state.item_class.vulgata_after_translated
          self.send(after_translated_callback, @translation_state.item) unless after_translated_callback.blank?

          format.html { redirect_to redirect_path, notice: 'Translation was successfully saved.' }
          format.json { render :show, status: :created, location: redirect_path }
        else
          format.html { redirect_to redirect_path, notice: 'Error, unable to create translation.' }
          format.json { render json: resource.errors, status: :unprocessable_entity }
        end
      end
    end

    def set_should_suggest_translation
      @should_suggest_translation = Vulgata.configuration.should_suggest_translation
    end

    def set_translation_state
      @translation_state = Vulgata::TranslationState.find(params[:translation_state_id])
    end

    def next_assigned_translation_state
      Vulgata::TranslationState
      .where(status: [Vulgata::TranslationState.statuses[:pending],
                      Vulgata::TranslationState.statuses[:in_progress]],
             user: vulgata_current_user,
             locale: @translator_locales)
      .order(translation_queue_sort)
      .first
    end

    def next_pending_translation_state
      Vulgata::TranslationState
      .where(status: Vulgata::TranslationState.statuses[:pending],
             user: nil,
             locale: @translator_locales).order(translation_queue_sort)
      .first
    end

    def dequeue_translation_state_for_translating
      Vulgata::TranslationState.transaction do
        translation_state = next_assigned_translation_state || next_pending_translation_state
        return nil if translation_state.nil?
        translation_state.status = Vulgata::TranslationState.statuses[:in_progress]
        translation_state.user = vulgata_current_user
        translation_state.save
        translation_state
      end
    end

    def set_translator_locales
      user_locales = Vulgata::UserLocale.where(user: vulgata_current_user) || []
      @translator_locales = user_locales.map(&:locale)
    end
  end
end
