module Vulgata
  class TranslationState < ActiveRecord::Base
    include Vulgata::StatusTypes

    belongs_to :item, polymorphic: true
    belongs_to :user, polymorphic: true
    has_many :versions, class_name: "Vulgata::TranslationStateVersion", foreign_key: :vulgata_translation_state_id

    attr_accessor :tr_data

    before_save :handle_comment
    after_save :save_version

    def status
      s = super
      s = s.to_sym unless s.nil?
    end

    def item_class
      @item_class ||= self.item_type.constantize
    end

    def source_translation_state
      @source_state ||= Vulgata::TranslationState.where(item_type: self.item_type, item_id: self.item_id, status: Vulgata::TranslationState.statuses[:source]).first
    end

    def item_class_name
      self.item.class.respond_to?(:vulgata_name) ? self.item.class.vulgata_name : self.item_type
    end

    def item_name
      self.item.respond_to?(:vulgata_name) ? self.item.vulgata_name.to_s.truncate(15, omission: "...") : self.item_id
    end

    def long_item_name
      self.item.respond_to?(:vulgata_name) ? self.item.vulgata_name.to_s.truncate(40, omission: "...") : self.item_id
    end

    def translation_data
      if ltv = latest_translated_version
        @translation ||= ltv.translation_data
      end

      @translation ||= self.item.vulgata_translation_data self
    end

    def preprocced_translation_data
      self.item.vulgata_preprocess_getter(translation_data)
    end

    def save_translation_version translation_data, translator
      return false if self.source?

      Vulgata::TranslationState.transaction do
        self.status = Vulgata::TranslationState.statuses[:done]
        self.user = translator if translator.present?
        self.updated_at = DateTime.now
        self.tr_data = self.item.vulgata_preprocess_setter(translation_data)
        self.save
      end
    end

    def approve_translation translator = nil
      transaction_succeeded = Vulgata::TranslationState.transaction do
        self.apply_current_version_to_item
        self.update(status: :approved, user: translator, locked: false)
      end

      self.item.vulgata_after_approved if transaction_succeeded

      transaction_succeeded
    end

    def pending_approval?
      self.status.to_sym == :done || self.status.to_sym == :proofreading
    end

    def latest_translated_version
      self.versions.where(status: Vulgata::TranslationState.statuses[:done]).where.not(translation_data: nil).order(id: :asc).last
    end

    def latest_rejected_version
      self.versions.where(status: Vulgata::TranslationState.statuses[:rejected]).order(id: :asc).last
    end

    def apply_current_version_to_item
      self.item.vlg_skip_pendify = true
      self.item.vulgata_save_translation(self)
    end

    # returns array of the translated attributes names
    def translated_attribute_names
      self.item.vulgata_translated_attribute_names
    end

    private

    def save_version
      saved_translation_data = nil
      word_count = 0

      # save wordcount and translation data only when submitting a new translation
      if self.tr_data.present? && self.status.to_sym == :done
        saved_translation_data = self.tr_data
        self.source_translation_state.translation_data.values.each { |att| word_count += att ? att.scan(/\w+/).size : 0 }
      end

      self.versions << Vulgata::TranslationStateVersion.new(status: self.status, priority: self.priority, user: self.user, translation_data: saved_translation_data, word_count: word_count.nonzero?, comment: self.comment)
    end

    def handle_comment
      unless self.comment_changed?
        self.comment = nil
      end
    end
  end
end
